"""
Backward compatibility shim for picomidi.aftertouch

This module is deprecated. Use picomidi.messages.aftertouch instead.
"""

import warnings

from picomidi.messages.aftertouch import Aftertouch

warnings.warn(
    "picomidi.aftertouch is deprecated; use picomidi.messages.aftertouch instead.",
    DeprecationWarning,
    stacklevel=2,
)

__all__ = ["Aftertouch"]



"""
Backward compatibility shim for picomidi.channel

This module is deprecated. Use picomidi.core.channel_legacy instead.
"""

import warnings

from picomidi.core.channel_legacy import MidiChannel

warnings.warn(
    "picomidi.channel is deprecated; use picomidi.core.channel_legacy instead.",
    DeprecationWarning,
    stacklevel=2,
)

__all__ = ["MidiChannel"]



"""
Backward compatibility shim for picomidi.song

This module is deprecated. Use picomidi.messages.song instead.
"""

import warnings

from picomidi.messages.song import Song

warnings.warn(
    "picomidi.song is deprecated; use picomidi.messages.song instead.",
    DeprecationWarning,
    stacklevel=2,
)

__all__ = ["Song"]



"""
Backward compatibility shim for picomidi.note

This module is deprecated. Use picomidi.messages.note instead.
"""

import warnings

from picomidi.messages.note import MidiNote

warnings.warn(
    "picomidi.note is deprecated; use picomidi.messages.note instead.",
    DeprecationWarning,
    stacklevel=2,
)

__all__ = ["MidiNote"]



"""
Midi values classes
"""


class MinValues:
    """Minimum values"""

    SIGNED_SIXTEEN_BIT = 0x8000


class MaxValues:
    """Maximum unsigned constants for various bit-widths."""

    FOUR_BIT = 0x0F  # 15, 4-bit max (unsigned)
    SEVEN_BIT = 0x7F  # 127, 7-bit max (standard MIDI data byte)
    EIGHT_BIT = 0xFF  # 255, 8-bit max (unsigned)
    FOURTEEN_BIT = 0x3FFF  # 16383, 14-bit max (used in MIDI pitch bend)
    SIXTEEN_BIT = 0xFFFF  # 65535, unsigned 16-bit full range
    THIRTY_TWO_BIT = 0xFFFFFFFF  # 4294967295, max unsigned 32-bit



"""
Backward compatibility shim for picomidi.value

This module is deprecated. Use picomidi.core.value instead.
"""

import warnings

from picomidi.core.value import MidiValue

warnings.warn(
    "picomidi.value is deprecated; use picomidi.core.value instead.",
    DeprecationWarning,
    stacklevel=2,
)

__all__ = ["MidiValue"]



"""
Backward compatibility shim for picomidi.tempo

This module is deprecated. Use picomidi.core.tempo instead.
"""

import warnings

from picomidi.core.tempo import MidiTempo

warnings.warn(
    "picomidi.tempo is deprecated; use picomidi.core.tempo instead.",
    DeprecationWarning,
    stacklevel=2,
)

__all__ = ["MidiTempo"]



"""
Standard MIDI Protocol Constants
"""

from picomidi.cc.nrpn import NonRegisteredParameterNumber
from picomidi.cc.rpn import RegisteredParameterNumber
from picomidi.core.channel_legacy import MidiChannel
from picomidi.core.tempo import MidiTempo
from picomidi.core.value import MidiValue
from picomidi.messages.aftertouch import Aftertouch
from picomidi.messages.control_change import ControlChange
from picomidi.messages.note import MidiNote
from picomidi.messages.pitch_bend import PitchBend
from picomidi.messages.program_change import ProgramChange
from picomidi.messages.song import Song
from picomidi.messages.sysex import MidiSysExByte
from picomidi.misc import MidiMisc


class Midi:
    """Standard MIDI protocol constants."""

    value = MidiValue
    note = MidiNote
    sysex = MidiSysExByte
    cc = ControlChange
    pc = ProgramChange
    aftertouch = Aftertouch
    song = Song
    pitch_bend = PitchBend
    tempo = MidiTempo
    channel = MidiChannel
    misc = MidiMisc
    nrpn = RegisteredParameterNumber
    rpn = NonRegisteredParameterNumber



"""Setup script for PicoMidi."""

from setuptools import find_packages, setup

# Read the README file for the long description
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# Read version from package
import re

with open("picomidi/__init__.py", "r", encoding="utf-8") as fh:
    version_match = re.search(r'^__version__ = ["\']([^"\']*)["\']', fh.read(), re.MULTILINE)
    if version_match:
        version = version_match.group(1)
    else:
        version = "0.1.0"

setup(
    name="picomidi",
    version=version,
    author="Mark Brooks",
    author_email="",  # Add your email if desired
    description="A lightweight, focused MIDI library for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/markxbrooks/picomidi",  # Update with your repository URL
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Multimedia :: Sound/Audio :: MIDI",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
    install_requires=[
        # No external dependencies - pure Python library
    ],
    extras_require={
        "dev": [
            "pytest>=7.0.0",
            "pytest-cov>=4.0.0",
            "black>=23.0.0",
            "flake8>=6.0.0",
            "mypy>=1.0.0",
        ],
    },
    include_package_data=True,
    zip_safe=False,
)



"""
MidiMisc
"""


class MidiMisc:
    """Misc Midi Values"""

    NOTES_NUMBER = 128  # Standard MIDI has 128 notes (0-127)
    TIME_CODE = 0xF1
    TUNE_REQUEST = 0xF6

    CLOCK = 0xF8
    ACTIVE_SENSING = 0xFE
    SYSTEM_RESET = 0xFF



"""
PicMidi - A lightweight MIDI library for Python

PicMidi provides core MIDI protocol functionality including:
- MIDI message creation and parsing
- Value conversions and validation
- Channel and status byte handling
- Timing and tempo calculations
"""

__version__ = "0.1.0"

# Constants
from picomidi.constant import Midi

# Core modules
from picomidi.core.bitmask import BitMask
from picomidi.core.channel import Channel
from picomidi.core.channel_legacy import MidiChannel
from picomidi.core.midistatus import MidiStatus
from picomidi.core.tempo import MidiTempo
from picomidi.core.types import ControlValue, Note, PitchBendValue, ProgramNumber, Velocity
from picomidi.core.value import MidiValue

# Message classes (new structured messages)
from picomidi.message.base import Message
from picomidi.message.channel_voice import (
    NRPN,
    RPN,
    ControlChange,
    NoteOff,
    NoteOn,
    PitchBend,
    ProgramChange,
)
from picomidi.message.sysex import RolandSysExMessage

# Message classes (legacy constants)
from picomidi.messages.aftertouch import Aftertouch
from picomidi.messages.control_change import ControlChange as ControlChangeStatus
from picomidi.messages.note import MidiNote
from picomidi.messages.pitch_bend import PitchBend as PitchBendStatus
from picomidi.messages.program_change import ProgramChange as ProgramChangeStatus
from picomidi.messages.song import Song
from picomidi.messages.sysex import MidiSysExByte

# Parser
from picomidi.parser.parser import Parser

# RPN/NRPN
from picomidi.rpn import NRPNMap, ParameterMap, RPNMap

# Utilities
from picomidi.utils import conversion, formatting, timing, validation

__all__ = [
    # Core
    "BitMask",
    "MidiStatus",
    "Channel",
    "MidiChannel",
    "MidiTempo",
    "MidiValue",
    # Types
    "Note",
    "Velocity",
    "ControlValue",
    "ProgramNumber",
    "PitchBendValue",
    # Messages (new structured)
    "Message",
    "NoteOn",
    "NoteOff",
    "ControlChange",
    "ProgramChange",
    "PitchBend",
    "RPN",
    "NRPN",
    "RolandSysExMessage",
    # Messages (legacy constants)
    "Aftertouch",
    "ControlChangeStatus",
    "MidiNote",
    "PitchBendStatus",
    "ProgramChangeStatus",
    "Song",
    "MidiSysExByte",
    # Constants
    "Midi",
    # Parser
    "Parser",
    # Utilities
    "conversion",
    "validation",
    "formatting",
    "timing",
    # RPN/NRPN
    "ParameterMap",
    "RPNMap",
    "NRPNMap",
]
